#ifndef I2C_ROTARY_003_01
#define I2C_ROTARY_003_01


#include <Wire.h>

class I2C_Rotary
{
public:

   // Id = 1 .. 3
   // Set default values:
   I2C_Rotary(byte NewAddress, byte NewId)
   {  m_Address = NewAddress;
      m_Id      = NewId;

      // Set defaults
      m_Value    =    0.0;
      m_MinValue =    0.0;
      m_MaxValue = 1000.0;
      m_StepSize =    1.0;
   }

   // Check Button Status
   //  - 0 : Not pressed
   //  - 1 : Short press
   //  - 2 : Long press
   byte Button()
   {  const byte NrBytes = 3;
      byte Response[NrBytes] = { 0 };
      if( ReadRegister((byte)(0xB0 + m_Id), Response, NrBytes) >= NrBytes )
      {  return Response[2];
      }
      else
      {  // Xmit failed for some reason. See m_XmitStatus.
         return 0;
      }
   }

   // Check rotation.
   // And update m_Value.
   int Clicks()
   {  const byte NrBytes = 5;
      byte Response[NrBytes] = { 0 };
      if( ReadRegister((byte)(0xC0 + m_Id), Response, NrBytes) >= NrBytes )
      {
         // Calculate NrClicks
         // Must be int16_t. Some Arduinos have 32-bit registers.
         int16_t NrClicks = (Response[2] << 8)
                          + (Response[3] << 0);

         // Update Value
         m_Value += NrClicks * m_StepSize;

         // Apply limits
         if(m_Value < m_MinValue)
         {  m_Value = m_MinValue;
         }
         if(m_Value > m_MaxValue)
         {  m_Value = m_MaxValue;
         }

         return NrClicks;
      }
      else
      {  // Xmit failed for some reason. See m_XmitStatus.
         return 0;
      }
   }

   // To set direction of rotary movement:
   //   direction = 0  : A leads B
   //   direction = 1  : B leads A
   void SetDirection(byte NewDirection)
   {  WriteRegister((byte)(0x10 + m_Id), NewDirection);   // MsgId : Set direction
      delay(20); // Target needs some time to update EEPROM
   }

   // To set accelleration of rotary movement:
   //   acceleration = 0  : No
   //   acceleration = 1  : Yes
   void SetAccelleration(byte Accelleration)
   {  WriteRegister((byte)(0x20 + m_Id), Accelleration);  // MsgId : Set Accelleration
      delay(20); // Target needs some time to update EEPROM
   }

   // To set a new I2C address
   // - But perhaps the slave has already switched to the new address
   byte ChangeAddress(byte OldAddress, byte NewAddress)
   {
      byte Response[3];  // Little buffer for response messages

      // First check new address
      m_Address = NewAddress;
      if( ReadRegister((byte)(0xF0), Response, 3) == 3 )   // MsgId : I2C address
      {  if(Response[2] == NewAddress)
         {  return 0;  // New adress already active
         }
         else
         {  return 1;  // Something is present on this address, but response is wrong
         }
      }
      else
      {  // No response on New address yet. Send update to old address
         m_Address = OldAddress;
         WriteRegister((byte)0xF0, NewAddress);

         // controller needs some time to update the EEprom
         delay(20);

         // Check if it hs worked
         m_Address = NewAddress;
         if(    (ReadRegister( (byte)0xF0, Response, 3 ) == 3)   // MsgId : I2C address
             && (Response[2] == NewAddress) )
         {  return 2;
         }
         else
         {  return 3;
         }
      }
   }

   // Send a message to the rotary controller.
   void WriteRegister(byte Register, byte Value)
   {  Wire.beginTransmission(m_Address >> 1);
      Wire.write(Register);
      Wire.write(Value);
      m_XmitStatus = Wire.endTransmission(true);
   }

   // Request some data from the rotary controller.
   byte ReadRegister(byte Register, byte *pBuffer, byte NrBytes)
   {  Wire.beginTransmission((byte)(m_Address >> 1));
      Wire.write(Register);
      m_XmitStatus = Wire.endTransmission(false);

      if(m_XmitStatus == 0)
      {  m_RecvCount = Wire.requestFrom((byte)(m_Address >> 1), NrBytes);
         for (byte i = 0; i < m_RecvCount; i++)
         {  pBuffer[i] = Wire.read();
         }
         return m_RecvCount;
      }
      else
      {  // Xmit failed for some reason.
         // - See m_XmitStatus.
         return 0;
      }
   }

   byte  m_Address;    // I2C address of rotary controller
   byte  m_Id;         // Switch Nr 1 .. 3

   float m_Value;      // Rotary status

   float m_MinValue;   // Limits
   float m_MaxValue;
   float m_StepSize;

   byte  m_XmitStatus; // Communicatio0n status
   byte  m_RecvCount;  //
};

#endif